# coding: utf-8

"""
    backend/api/experiment.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import kfp_experiment
from kfp_experiment.api.experiment_service_api import ExperimentServiceApi  # noqa: E501
from kfp_experiment.rest import ApiException


class TestExperimentServiceApi(unittest.TestCase):
    """ExperimentServiceApi unit test stubs"""

    def setUp(self):
        self.api = kfp_experiment.api.experiment_service_api.ExperimentServiceApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_experiment(self):
        """Test case for create_experiment

        """
        pass

    def test_delete_experiment(self):
        """Test case for delete_experiment

        """
        pass

    def test_get_experiment(self):
        """Test case for get_experiment

        """
        pass

    def test_list_experiment(self):
        """Test case for list_experiment

        """
        pass


if __name__ == '__main__':
    unittest.main()
