# coding: utf-8

"""
    backend/api/run.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import kfp_run
from kfp_run.models.run_metric_format import RunMetricFormat  # noqa: E501
from kfp_run.rest import ApiException


class TestRunMetricFormat(unittest.TestCase):
    """RunMetricFormat unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testRunMetricFormat(self):
        """Test RunMetricFormat"""
        # FIXME: construct object with mandatory attributes with example values
        # model = kfp_run.models.run_metric_format.RunMetricFormat()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
