# coding: utf-8

"""
    backend/api/pipeline.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import kfp_pipeline
from kfp_pipeline.api.pipeline_service_api import PipelineServiceApi  # noqa: E501
from kfp_pipeline.rest import ApiException


class TestPipelineServiceApi(unittest.TestCase):
    """PipelineServiceApi unit test stubs"""

    def setUp(self):
        self.api = kfp_pipeline.api.pipeline_service_api.PipelineServiceApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_pipeline(self):
        """Test case for create_pipeline

        """
        pass

    def test_delete_pipeline(self):
        """Test case for delete_pipeline

        """
        pass

    def test_get_pipeline(self):
        """Test case for get_pipeline

        """
        pass

    def test_get_template(self):
        """Test case for get_template

        """
        pass

    def test_list_pipelines(self):
        """Test case for list_pipelines

        """
        pass


if __name__ == '__main__':
    unittest.main()
