# coding: utf-8

"""
    backend/api/job.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import kfp_job
from kfp_job.api.job_service_api import JobServiceApi  # noqa: E501
from kfp_job.rest import ApiException


class TestJobServiceApi(unittest.TestCase):
    """JobServiceApi unit test stubs"""

    def setUp(self):
        self.api = kfp_job.api.job_service_api.JobServiceApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_job(self):
        """Test case for create_job

        """
        pass

    def test_delete_job(self):
        """Test case for delete_job

        """
        pass

    def test_disable_job(self):
        """Test case for disable_job

        """
        pass

    def test_enable_job(self):
        """Test case for enable_job

        """
        pass

    def test_get_job(self):
        """Test case for get_job

        """
        pass

    def test_list_jobs(self):
        """Test case for list_jobs

        """
        pass


if __name__ == '__main__':
    unittest.main()
