# coding: utf-8

"""
    pipeline upload

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import kfp_uploadpipeline
from kfp_uploadpipeline.api.pipeline_upload_service_api import PipelineUploadServiceApi  # noqa: E501
from kfp_uploadpipeline.rest import ApiException


class TestPipelineUploadServiceApi(unittest.TestCase):
    """PipelineUploadServiceApi unit test stubs"""

    def setUp(self):
        self.api = kfp_uploadpipeline.api.pipeline_upload_service_api.PipelineUploadServiceApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_upload_pipeline(self):
        """Test case for upload_pipeline

        """
        pass


if __name__ == '__main__':
    unittest.main()
