# coding: utf-8

"""
    backend/api/experiment.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import unittest

import kfp_experiment
from kfp_experiment.models.api_status import ApiStatus  # noqa: E501
from kfp_experiment.rest import ApiException


class TestApiStatus(unittest.TestCase):
    """ApiStatus unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testApiStatus(self):
        """Test ApiStatus"""
        # FIXME: construct object with mandatory attributes with example values
        # model = kfp_experiment.models.api_status.ApiStatus()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
